/* 
/ spatialite.h -- SpatiaLite - spatial extension for Sqlite
/  
/ version 1.0, 2007 October 25
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2007  Alessandro Furieri
/
/ This library is free software; you can redistribute it and/or
/ modify it under the terms of the GNU Lesser General Public
/ License as published by the Free Software Foundation; either
/ version 2.1 of the License, or (at your option) any later version.
/
/ This library is distributed in the hope that it will be useful,
/ but WITHOUT ANY WARRANTY; without even the implied warranty of
/ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/ Lesser General Public License for more details.
/
/ You should have received a copy of the GNU Lesser General Public
/ License along with this library; if not, write to the Free Software
/ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA/
*/

#ifdef WIN32_DLL
#define SPATIALITE_DECLARE __declspec(dllexport)
#else
#define SPATIALITE_DECLARE extern
#endif

#ifndef _SPATIALITE_H
#define _SPATIALITE_H

#ifdef __cplusplus
extern "C"
{
#endif

/*
/ constants that defines GEOMETRY CLASSes
*/
#define SPLE_UNKNOWN			0
#define SPLE_POINT				1
#define SPLE_LINESTRING			2
#define SPLE_POLYGON			3
#define SPLE_MULTIPOINT			4
#define SPLE_MULTILINESTRING	5
#define SPLE_MULTIPOLYGON		6
#define SPLE_GEOMETRYCOLLECTION	7

/*
/ constants that defines token codes for WKT parsing
*/
#define SPLE_COORDINATE			8
#define SPLE_OPENED				9
#define SPLE_CLOSED				10
#define SPLE_COMMA				11
#define SPLE_SPACE				12

/*
/ constants that defines request mode for point_n() [StartPoint(), EndPoint(), PointN()]
*/
#define SPLE_START_POINT		1
#define SPLE_END_POINT			2
#define SPLE_POINTN				3

/*
/ constants that defines request mode for ring_n() [ExteriorRing(), InteriorRingN()]
*/
#define SPLE_EXTERIOR_RING		1
#define SPLE_INTERIOR_RING		2

/*
/ constants that defines byte storage order 
*/
#define SPLE_BIG_ENDIAN		0x00
#define SPLE_LITTLE_ENDIAN	0x01

/*
/ constants that defines special markers used for encoding of internal BLOB geometries 
*/
#define SPLE_MARK_START		0x00
#define SPLE_MARK_END		0xFE
#define SPLE_MARK_MBR		0x7C
#define SPLE_MARK_ENTITY	0x69

/*
/ constants that defines DBF Field types
*/
#define SPLE_DBF_CHAR	'C'
#define SPLE_DBF_NUM	'N'
#define SPLE_DBF_BOOL	'L'
#define SPLE_DBF_DATE	'D'

/*
/ constants that defines MBRs spatial relationships
*/
#define SPLE_MBR_CONTAINS	1
#define SPLE_MBR_DISJOINT	2
#define SPLE_MBR_EQUAL		3
#define SPLE_MBR_INTERSECTS	4
#define SPLE_MBR_OVERLAPS	5
#define SPLE_MBR_TOUCHES	6
#define SPLE_MBR_WITHIN		7

    typedef struct sple_point_struct
    {
	/* an OpenGis POINT */
	double x;		/* X,Y coordinates */
	double y;
    } sple_point;
    typedef sple_point *sple_point_ptr;

    typedef struct sple_linestring_struct
    {
	/* an OpenGis LINESTRING */
	int points;		/* number of vertices */
	sple_point_ptr geometry;	/* pointer referencing an array of POINTs */
	double minx;		/* MBR - minimun x */
	double miny;		/* MBR - minimun y */
	double maxx;		/* MBR - maximun x */
	double maxy;		/* MBR - maximun y */
    } sple_linestring;
    typedef sple_linestring *sple_linestring_ptr;

    typedef struct sple_polygon_struct
    {
	/* an OpenGis POLYGON */
	sple_linestring_ptr external;	/* pointer referenging the polygon's EXTERNAL border */
	int borders;		/* number of interneal borders */
	sple_linestring_ptr *internals;	/* pointer referencing an array of polygon's INTERNAL borders [may be null ... if no internal border exists] */
	double minx;		/* MBR - minimun x */
	double miny;		/* MBR - minimun y */
	double maxx;		/* MBR - maximun x */
	double maxy;		/* MBR - maximun y */
    } sple_polygon;
    typedef sple_polygon *sple_polygon_ptr;

    typedef struct sple_generic_geometry_struct
    {
	/* a generic container for OpenGis GEOMETRIES */
	short type;		/* geometry type [may be SPLE_POINT,SPLE_LINESTRING or SPLE_POLYGON] */
	void *geometry;
	/* should be safely casted to *sple_point_ptr if *type* = SPLE_POINT
	   / to *sple_linestring_ptr* if *type* = SPLE_LINESTRING 
	   / or to *sple_polygon_ptr* if *type* = SPLE_POLYGON
	 */
    } sple_generic_geometry;
    typedef sple_generic_geometry *sple_generic_geometry_ptr;

    typedef struct sple_geometry_struct
    {
	/* an OpenGis GEOMETRY COLLECTION */
	int little_endian;	/* TRUE if buffer uses little-endian encoding */
	const unsigned char *blob;	/* pointer referencing the WKB encoded buffer */
	int size;		/* buffer size */
	int offset;		/* current offset used by parsing routines] */
	int errors;		/* error count [updated by parsing routines] */
	int entities;		/* number of entities */
	sple_generic_geometry_ptr *geometries;	/* pointer referenging an array of generic geometries */
	double minx;		/* MBR - minimun x */
	double miny;		/* MBR - minimun y */
	double maxx;		/* MBR - maximun x */
	double maxy;		/* MBR - maximun y */
    } sple_geometry;
    typedef sple_geometry *sple_geometry_ptr;

    typedef struct sple_token_struct
    {
	/* linked list of pre-parsed tokens - used in WKT parsing */
	int type;		/* token code */
	double coord;		/* a coordinate [if any] */
	struct sple_token_struct *next;	/* reference to next element in linked list */
    } sple_token;
    typedef sple_token *sple_token_ptr;

    typedef struct sple_list_token_struct
    {
	/* 
	   / a block of consecutive WKT tokens that identify a LINESTRING or RING, 
	   / in the form: (1 2, 3 4, 5 6, 7 8)
	 */
	sple_token_ptr first;	/* reference to first element in linked list - has always to be a '(' */
	sple_token_ptr last;	/* reference to last element in linked list - has always to be a ')' */
	int points;		/* number of POINTS contained in the linkek list */
	sple_linestring_ptr line;	/* the LINESTRING builded after successful parsing */
	struct sple_list_token_struct *next;	/* points to next element in linked list [if any] */
    } sple_list_token;
    typedef sple_list_token *sple_list_token_ptr;

    typedef struct sple_multi_list_token_struct
    {
	/*
	   / a group of the above token lists, that identify a POLYGON or a MULTILINESTRING
	   / in the form: ((...),(...),(...))
	 */
	sple_list_token_ptr first;	/* reference to first element in linked list - has always to be a '(' */
	sple_list_token_ptr last;	/* reference to last element in linked list - has always to be a ')' */
	struct sple_multi_list_token_struct *next;	/* points to next element in linked list [if any] */
    } sple_multi_list_token;
    typedef sple_multi_list_token *sple_multi_list_token_ptr;

    typedef struct sple_multi_multi_list_token_struct
    {
	/*
	   / a group of the above token multi-lists, that identify a MULTIPOLYGON 
	   / in the form: (((...),(...),(...)),((...),(...)),((...)))
	 */
	sple_multi_list_token_ptr first;	/* reference to first element in linked list - has always to be a '(' */
	sple_multi_list_token_ptr last;	/* reference to last element in linked list - has always to be a ')' */
    } sple_multi_multi_list_token;
    typedef sple_multi_multi_list_token *sple_multi_multi_list_token_ptr;

    typedef struct sple_var_list_token_struct
    {
	/* 
	   / a multitype variable reference that may be associated to:
	   / - a POINT element
	   / - a LINESTRING element
	   / - a POLYGON element
	 */
	int type;		/* may be SPLE_POINT, SPLE_LINESTRING or SPLE_POLYGON */
	void *pointer;		/* 
				   / to be casted as *sple_multi_list_token_ptr* if type is SPLE_LINESTRING
				   / or as *sple_multi_multi_list_token_ptr* if type is SPLE_POLYGON
				 */
	double x;		/* X,Y are valids only if type is SPLE_POINT */
	double y;
	struct sple_var_list_token_struct *next;	/* points to next element in linked list */
    } sple_var_list_token;
    typedef sple_var_list_token *sple_var_list_token_ptr;

    typedef struct sple_geocoll_list_token_struct
    {
	/*
	   / a group of lists and multi-lists that identifies a GEOMETRYCOLLECTION 
	   / in the form: (ELEM(...),ELEM(...),ELEM(....))
	 */
	sple_var_list_token_ptr first;	/* reference to first element in linked list - has always to be a '(' */
	sple_var_list_token_ptr last;	/* reference to last element in linked list - has always to be a ')' */
    } sple_geocoll_list_token;
    typedef sple_geocoll_list_token *sple_geocoll_list_token_ptr;

    typedef struct sple_dbf_field_struct
    {
	/* a DBF field definition - shapefile attribute */
	char *name;		/* field name */
	unsigned char type;	/* field type */
	int offset;		/* buffer offset [this field begins at *buffer+offset* and extends for *length* bytes */
	unsigned char length;	/* field total lenght [in bytes] */
	unsigned char decimals;	/* decimal positions */
	struct sple_dbf_field_struct *next;	/* pointer to next element in linker list */
    } sple_dbf_field;
    typedef sple_dbf_field *sple_dbf_field_ptr;

    typedef struct sple_dbf_list_struct
    {
	/* a linked list to containt the DBF fields definitions - shapefile attributes */
	char *primary_key_name;	/* name of column used as primary key in Sqlite table */
	char *geometry_name;	/* name of column used as geometry in Sqlite table */
	sple_dbf_field_ptr first;	/* pointer to first element in linked list */
	sple_dbf_field_ptr last;	/* pointer to last element in linked list */
	int seed;		/* a counter for anti-aliased column names */
    } sple_dbf_list;
    typedef sple_dbf_list *sple_dbf_list_ptr;

    typedef struct sple_dbf_export_struct
    {
	/* a DBF field definition - export shapefile attribute */
	char *name;		/* field name */
	/*
	   / those variables are used in the preliminary data fetching
	   / to compute real size and type for each DBF attribute
	 */
	int sql_type;		/* SQL column type */
	int max_length;		/* SQL data max length */
	int boolean;		/* TRUE [1] if this column has only BOOLEAN values */
	int date;		/* TRUE [1] if this column has only DATE values */
	/*
	   / those variables are used only when generating DBF file
	   / and are set by sple_compute_export_dbf()
	 */
	char dbf_type;		/* DBF field type */
	int offset;		/* buffer offset [this field begins at *buffer+offset* and extends for *length* bytes */
	unsigned char length;	/* field total lenght [in bytes] */
	unsigned char decimals;	/* decimal positions */
    } sple_dbf_export;
    typedef sple_dbf_export *sple_dbf_export_ptr;

/* pseudo SQL functions - import/export tools */
    SPATIALITE_DECLARE void LoadShapefile (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void DumpShapefile (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
/* SQL Spatial functions - OpenGis compliant - */
    SPATIALITE_DECLARE void AsText (sqlite3_context * context, int argc,
				    sqlite3_value ** argv);
    SPATIALITE_DECLARE void AsBinary (sqlite3_context * context, int argc,
				      sqlite3_value ** argv);
    SPATIALITE_DECLARE void GeomFromText1 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void GeomFromText2 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void GeomCollFromText1 (sqlite3_context * context,
					       int argc, sqlite3_value ** argv);
    SPATIALITE_DECLARE void GeomCollFromText2 (sqlite3_context * context,
					       int argc, sqlite3_value ** argv);
    SPATIALITE_DECLARE void LineFromText1 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void LineFromText2 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void PointFromText1 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv);
    SPATIALITE_DECLARE void PointFromText2 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv);
    SPATIALITE_DECLARE void PolyFromText1 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void PolyFromText2 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void MLineFromText1 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv);
    SPATIALITE_DECLARE void MLineFromText2 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv);
    SPATIALITE_DECLARE void MPointFromText1 (sqlite3_context * context,
					     int argc, sqlite3_value ** argv);
    SPATIALITE_DECLARE void MPointFromText2 (sqlite3_context * context,
					     int argc, sqlite3_value ** argv);
    SPATIALITE_DECLARE void MPolyFromText1 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv);
    SPATIALITE_DECLARE void MPolyFromText2 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv);
    SPATIALITE_DECLARE void geom_from_text1 (sqlite3_context * context,
					     int argc, sqlite3_value ** argv,
					     short type);
    SPATIALITE_DECLARE void geom_from_text2 (sqlite3_context * context,
					     int argc, sqlite3_value ** argv,
					     short type);
    SPATIALITE_DECLARE void GeomFromWKB1 (sqlite3_context * context, int argc,
					  sqlite3_value ** argv);
    SPATIALITE_DECLARE void GeomFromWKB2 (sqlite3_context * context, int argc,
					  sqlite3_value ** argv);
    SPATIALITE_DECLARE void GeomCollFromWKB1 (sqlite3_context * context,
					      int argc, sqlite3_value ** argv);
    SPATIALITE_DECLARE void GeomCollFromWKB2 (sqlite3_context * context,
					      int argc, sqlite3_value ** argv);
    SPATIALITE_DECLARE void LineFromWKB1 (sqlite3_context * context, int argc,
					  sqlite3_value ** argv);
    SPATIALITE_DECLARE void LineFromWKB2 (sqlite3_context * context, int argc,
					  sqlite3_value ** argv);
    SPATIALITE_DECLARE void PointFromWKB1 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void PointFromWKB2 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void PolyFromWKB1 (sqlite3_context * context, int argc,
					  sqlite3_value ** argv);
    SPATIALITE_DECLARE void PolyFromWKB2 (sqlite3_context * context, int argc,
					  sqlite3_value ** argv);
    SPATIALITE_DECLARE void MLineFromWKB1 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void MLineFromWKB2 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void MPointFromWKB1 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv);
    SPATIALITE_DECLARE void MPointFromWKB2 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv);
    SPATIALITE_DECLARE void MPolyFromWKB1 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void MPolyFromWKB2 (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void geom_from_wkb1 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv, short type);
    SPATIALITE_DECLARE void geom_from_wkb2 (sqlite3_context * context, int argc,
					    sqlite3_value ** argv, short type);
    SPATIALITE_DECLARE void Dimension (sqlite3_context * context, int argc,
				       sqlite3_value ** argv);
    SPATIALITE_DECLARE void GeometryType (sqlite3_context * context, int argc,
					  sqlite3_value ** argv);
    SPATIALITE_DECLARE void Srid (sqlite3_context * context, int argc,
				  sqlite3_value ** argv);
    SPATIALITE_DECLARE void Envelope (sqlite3_context * context, int argc,
				      sqlite3_value ** argv);
    SPATIALITE_DECLARE void X (sqlite3_context * context, int argc,
			       sqlite3_value ** argv);
    SPATIALITE_DECLARE void Y (sqlite3_context * context, int argc,
			       sqlite3_value ** argv);
    SPATIALITE_DECLARE void EndPoint (sqlite3_context * context, int argc,
				      sqlite3_value ** argv);
    SPATIALITE_DECLARE void StartPoint (sqlite3_context * context, int argc,
					sqlite3_value ** argv);
    SPATIALITE_DECLARE void PointN (sqlite3_context * context, int argc,
				    sqlite3_value ** argv);
    SPATIALITE_DECLARE void point_n (sqlite3_context * context, int argc,
				     sqlite3_value ** argv, int request);
    SPATIALITE_DECLARE void IsEmpty (sqlite3_context * context, int argc,
				     sqlite3_value ** argv);
    SPATIALITE_DECLARE void IsClosed (sqlite3_context * context, int argc,
				      sqlite3_value ** argv);
    SPATIALITE_DECLARE void IsSimple (sqlite3_context * context, int argc,
				      sqlite3_value ** argv);
    SPATIALITE_DECLARE void IsRing (sqlite3_context * context, int argc,
				    sqlite3_value ** argv);
    SPATIALITE_DECLARE void NumPoints (sqlite3_context * context, int argc,
				       sqlite3_value ** argv);
    SPATIALITE_DECLARE void Length (sqlite3_context * context, int argc,
				    sqlite3_value ** argv);
    SPATIALITE_DECLARE void Area (sqlite3_context * context, int argc,
				  sqlite3_value ** argv);
    SPATIALITE_DECLARE void Centroid (sqlite3_context * context, int argc,
				      sqlite3_value ** argv);
    SPATIALITE_DECLARE void ExteriorRing (sqlite3_context * context, int argc,
					  sqlite3_value ** argv);
    SPATIALITE_DECLARE void InteriorRingN (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void ring_n (sqlite3_context * context, int argc,
				    sqlite3_value ** argv, int request);
    SPATIALITE_DECLARE void NumInteriorRings (sqlite3_context * context,
					      int argc, sqlite3_value ** argv);
    SPATIALITE_DECLARE void NumGeometries (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void GeometryN (sqlite3_context * context, int argc,
				       sqlite3_value ** argv);
    SPATIALITE_DECLARE void MBRContains (sqlite3_context * context, int argc,
					 sqlite3_value ** argv);
    SPATIALITE_DECLARE void MBRDisjoint (sqlite3_context * context, int argc,
					 sqlite3_value ** argv);
    SPATIALITE_DECLARE void MBREqual (sqlite3_context * context, int argc,
				      sqlite3_value ** argv);
    SPATIALITE_DECLARE void MBRIntersects (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void MBROverlaps (sqlite3_context * context, int argc,
					 sqlite3_value ** argv);
    SPATIALITE_DECLARE void MBRTouches (sqlite3_context * context, int argc,
					sqlite3_value ** argv);
    SPATIALITE_DECLARE void MBRWithin (sqlite3_context * context, int argc,
				       sqlite3_value ** argv);
    SPATIALITE_DECLARE void mbrs_eval (sqlite3_context * context, int argc,
				       sqlite3_value ** argv, int request);
/* SQL Spatial functions - SpatiaLite owns - */
    SPATIALITE_DECLARE void ShiftCoords (sqlite3_context * context, int argc,
					 sqlite3_value ** argv);
    SPATIALITE_DECLARE void ScaleCoords (sqlite3_context * context, int argc,
					 sqlite3_value ** argv);
    SPATIALITE_DECLARE void RotateCoords (sqlite3_context * context, int argc,
					  sqlite3_value ** argv);
    SPATIALITE_DECLARE void ReflectCoords (sqlite3_context * context, int argc,
					   sqlite3_value ** argv);
    SPATIALITE_DECLARE void Transform (sqlite3_context * context, int argc,
				       sqlite3_value ** argv);
    SPATIALITE_DECLARE void SetSrid (sqlite3_context * context, int argc,
				     sqlite3_value ** argv);

/* internal helper methods */
    SPATIALITE_DECLARE unsigned short sple_import_16 (const unsigned char *p,
						      int little_endian);
    SPATIALITE_DECLARE unsigned int sple_import_32 (const unsigned char *p,
						    int little_endian);
    SPATIALITE_DECLARE double sple_import_64 (const unsigned char *p,
					      int little_endian);
    SPATIALITE_DECLARE void sple_export_16 (unsigned char *p,
					    unsigned short value,
					    int little_endian);
    SPATIALITE_DECLARE void sple_export_32 (unsigned char *p,
					    unsigned int value,
					    int little_endian);
    SPATIALITE_DECLARE void sple_export_64 (unsigned char *p, double value,
					    int little_endian);
    SPATIALITE_DECLARE sple_geometry_ptr sple_alloc_geometry ();
    SPATIALITE_DECLARE void sple_free_geometry (sple_geometry_ptr p);
    SPATIALITE_DECLARE sple_point_ptr sple_alloc_point (double x, double y);
    SPATIALITE_DECLARE void sple_free_point (sple_point_ptr p);
    SPATIALITE_DECLARE sple_linestring_ptr
	sple_alloc_internal_border (sple_polygon_ptr polyg, int position,
				    int vert);
    SPATIALITE_DECLARE sple_linestring_ptr sple_alloc_linestring (int vert);
    SPATIALITE_DECLARE void sple_free_linestring (sple_linestring_ptr p);
    SPATIALITE_DECLARE sple_polygon_ptr sple_alloc_polygon (int ext_vert,
							    int int_borders);
    SPATIALITE_DECLARE void sple_free_polygon (sple_polygon_ptr p);
    SPATIALITE_DECLARE sple_generic_geometry_ptr
	sple_alloc_generic_geometry (short type, void *ptr);
    SPATIALITE_DECLARE void
	sple_free_generic_geometry (sple_generic_geometry_ptr p);
    SPATIALITE_DECLARE sple_geometry_ptr sple_blob_decode (const unsigned char
							   *blob, int size);
    SPATIALITE_DECLARE sple_geometry_ptr sple_mbr_decode (const unsigned char
							  *blob, int size);
    SPATIALITE_DECLARE void sple_out_wkt (sple_geometry_ptr, char **buffer);
    SPATIALITE_DECLARE void sple_decode_point (sple_geometry_ptr p,
					       int position);
    SPATIALITE_DECLARE void sple_decode_linestring (sple_geometry_ptr p,
						    int position);
    SPATIALITE_DECLARE void sple_decode_polygon (sple_geometry_ptr p,
						 int position);
    SPATIALITE_DECLARE void sple_decode_geometry (sple_geometry_ptr p);
    SPATIALITE_DECLARE void sple_set_point (sple_linestring_ptr, int vert,
					    double x, double y);
    SPATIALITE_DECLARE short sple_multi_type (sple_geometry_ptr p);
    SPATIALITE_DECLARE void sple_out_text (char *txt, char **buffer, int *size);
    SPATIALITE_DECLARE void sple_out_point (sple_point_ptr point, char **buffer,
					    int *size);
    SPATIALITE_DECLARE void sple_out_linestring (sple_linestring_ptr line,
						 char **buffer, int *size);
    SPATIALITE_DECLARE void sple_out_polygon (sple_polygon_ptr polygon,
					      char **buffer, int *size);
    SPATIALITE_DECLARE void sple_out_check_buffer (char **buffer, int *size);
    SPATIALITE_DECLARE void sple_out_clean (char *buffer);
    SPATIALITE_DECLARE void sple_blob_to_wkb (sple_geometry_ptr p,
					      unsigned char **buffer,
					      int *size);
    SPATIALITE_DECLARE int sple_dimension (sple_geometry_ptr geom);
    SPATIALITE_DECLARE void sple_geometry_type (sple_geometry_ptr geom,
						char **buffer);
    SPATIALITE_DECLARE void sple_envelope (sple_geometry_ptr geom,
					   unsigned char **buffer, int *size);
    SPATIALITE_DECLARE int sple_x (sple_geometry_ptr geom, double *coord);
    SPATIALITE_DECLARE int sple_y (sple_geometry_ptr geom, double *coord);
    SPATIALITE_DECLARE int sple_num_points (sple_geometry_ptr geom,
					    int *points);
    SPATIALITE_DECLARE void sple_point_n (sple_geometry_ptr geom, int vertex,
					  unsigned char **buffer, int *size);
    SPATIALITE_DECLARE int sple_num_interior_rings (sple_geometry_ptr geom,
						    int *borders);
    SPATIALITE_DECLARE void sple_ring_n (sple_geometry_ptr geom, int border,
					 unsigned char **buffer, int *size);
    SPATIALITE_DECLARE int sple_num_geometries (sple_geometry_ptr geom,
						int *geoms);
    SPATIALITE_DECLARE void sple_geometry_n (sple_geometry_ptr geom, int entity,
					     unsigned char **buffer, int *size);
    SPATIALITE_DECLARE int sple_count_geometries (sple_geometry_ptr geom);
    SPATIALITE_DECLARE int sple_is_closed (sple_geometry_ptr geom);
    SPATIALITE_DECLARE int sple_is_simple (sple_geometry_ptr geom);
    SPATIALITE_DECLARE int sple_is_ring (sple_geometry_ptr geom);
    SPATIALITE_DECLARE int sple_length (sple_geometry_ptr geom, double *length);
    SPATIALITE_DECLARE int sple_area (sple_geometry_ptr geom, double *area);
    SPATIALITE_DECLARE void sple_centroid (sple_geometry_ptr geom,
					   unsigned char **buffer, int *size);
    SPATIALITE_DECLARE void sple_shift_coords (sple_geometry_ptr geom,
					       double shift_x, double shift_y,
					       unsigned char **buffer,
					       int *size);
    SPATIALITE_DECLARE void sple_scale_coords (sple_geometry_ptr geom,
					       double scale_x, double scale_y,
					       unsigned char **buffer,
					       int *size);
    SPATIALITE_DECLARE void sple_rotate_coords (sple_geometry_ptr geom,
						double angle,
						unsigned char **buffer,
						int *size);
    SPATIALITE_DECLARE void sple_reflect_coords (sple_geometry_ptr geom,
						 int x_axis, int y_axis,
						 unsigned char **buffer,
						 int *size);
    SPATIALITE_DECLARE void sple_as_binary (sple_geometry_ptr geom,
					    unsigned char **buffer, int *size);
    SPATIALITE_DECLARE char *sple_clean_wkt (const unsigned char *p);
    SPATIALITE_DECLARE sple_geometry_ptr sple_parse_wkt (char *wkt, short type);
    SPATIALITE_DECLARE sple_geometry_ptr sple_parse_wkb (const unsigned char
							 *wkb, int size,
							 short type);
    SPATIALITE_DECLARE void sple_add_token (char *token, sple_token_ptr * first,
					    sple_token_ptr * last);
    SPATIALITE_DECLARE int sple_parse_double (char *token, double *coord);
    SPATIALITE_DECLARE sple_point_ptr sple_build_point (sple_token_ptr first);
    SPATIALITE_DECLARE sple_geometry_ptr
	sple_geometry_from_point (sple_point_ptr point);
    SPATIALITE_DECLARE sple_geometry_ptr
	sple_geometry_from_mpoint (sple_linestring_ptr mpoint);
    SPATIALITE_DECLARE sple_geometry_ptr
	sple_geometry_from_linestring (sple_linestring_ptr line);
    SPATIALITE_DECLARE sple_geometry_ptr
	sple_geometry_from_mline (sple_multi_list_token_ptr mline);
    SPATIALITE_DECLARE sple_geometry_ptr
	sple_geometry_from_polygon (sple_multi_list_token_ptr polyg);
    SPATIALITE_DECLARE sple_geometry_ptr
	sple_geometry_from_mpoly (sple_multi_multi_list_token_ptr mpoly);
    SPATIALITE_DECLARE sple_geometry_ptr
	sple_geometry_from_geocoll (sple_geocoll_list_token_ptr geocoll);
    SPATIALITE_DECLARE void sple_blob_encode (sple_geometry_ptr geom,
					      unsigned char **buffer,
					      int *size);
    SPATIALITE_DECLARE void sple_mbr_geometry (sple_geometry_ptr geom);
    SPATIALITE_DECLARE void sple_mbr_linestring (sple_linestring_ptr line);
    SPATIALITE_DECLARE void sple_mbr_polygon (sple_polygon_ptr polyg);
    SPATIALITE_DECLARE sple_list_token_ptr sple_build_list_token (sple_token_ptr
								  stat,
								  sple_token_ptr
								  end);
    SPATIALITE_DECLARE sple_multi_list_token_ptr
	sple_build_multi_list_token (sple_token_ptr start, sple_token_ptr end);
    SPATIALITE_DECLARE sple_multi_multi_list_token_ptr
	sple_build_multi_multi_list_token (sple_token_ptr start,
					   sple_token_ptr end);
    SPATIALITE_DECLARE sple_geocoll_list_token_ptr
	sple_build_geocoll_list_token (sple_token_ptr start,
				       sple_token_ptr end);
    SPATIALITE_DECLARE void sple_free_list_token (sple_list_token_ptr p);
    SPATIALITE_DECLARE void
	sple_free_multi_list_token (sple_multi_list_token_ptr p);
    SPATIALITE_DECLARE void
	sple_free_multi_multi_list_token (sple_multi_multi_list_token_ptr p);
    SPATIALITE_DECLARE void
	sple_free_geocoll_list_token (sple_geocoll_list_token_ptr p);
    SPATIALITE_DECLARE int sple_is_simple_line (sple_linestring_ptr line);
    SPATIALITE_DECLARE int sple_are_simple_lines (sple_linestring_ptr line,
						  sple_linestring_ptr line2);
    SPATIALITE_DECLARE int sple_is_intersection (double ox1, double oy1,
						 double x1, double y1,
						 double ox2, double oy2,
						 double x2, double y2,
						 double *x, double *y);
    SPATIALITE_DECLARE int sple_point_in_line (sple_point_ptr point,
					       sple_linestring_ptr line);
    SPATIALITE_DECLARE int sple_contains_mbrs (double mbr1_minx,
					       double mbr1_miny,
					       double mbr1_maxx,
					       double mbr1_maxy,
					       double mbr2_minx,
					       double mbr2_miny,
					       double mbr2_maxx,
					       double mbr2_maxy);
    SPATIALITE_DECLARE int sple_disjoint_mbrs (double mbr1_minx,
					       double mbr1_miny,
					       double mbr1_maxx,
					       double mbr1_maxy,
					       double mbr2_minx,
					       double mbr2_miny,
					       double mbr2_maxx,
					       double mbr2_maxy);
    SPATIALITE_DECLARE int sple_equal_mbrs (double mbr1_minx, double mbr1_miny,
					    double mbr1_maxx, double mbr1_maxy,
					    double mbr2_minx, double mbr2_miny,
					    double mbr2_maxx, double mbr2_maxy);
    SPATIALITE_DECLARE int sple_intersects_mbrs (double mbr1_minx,
						 double mbr1_miny,
						 double mbr1_maxx,
						 double mbr1_maxy,
						 double mbr2_minx,
						 double mbr2_miny,
						 double mbr2_maxx,
						 double mbr2_maxy);
    SPATIALITE_DECLARE int sple_touches_mbrs (double mbr1_minx,
					      double mbr1_miny,
					      double mbr1_maxx,
					      double mbr1_maxy,
					      double mbr2_minx,
					      double mbr2_miny,
					      double mbr2_maxx,
					      double mbr2_maxy);
    SPATIALITE_DECLARE int sple_overlaps_mbrs (double mbr1_minx,
					       double mbr1_miny,
					       double mbr1_maxx,
					       double mbr1_maxy,
					       double mbr2_minx,
					       double mbr2_miny,
					       double mbr2_maxx,
					       double mbr2_maxy);
    SPATIALITE_DECLARE int sple_within_mbrs (double mbr1_minx, double mbr1_miny,
					     double mbr1_maxx, double mbr1_maxy,
					     double mbr2_minx, double mbr2_miny,
					     double mbr2_maxx,
					     double mbr2_maxy);
    SPATIALITE_DECLARE int sple_load_shapefile (const unsigned char *path,
						const unsigned char *table,
						const unsigned char *column,
						int srid, int *entities,
						int *shape, char **error);
    SPATIALITE_DECLARE int sple_dump_shapefile (const unsigned char *path,
						const unsigned char *table,
						const unsigned char *column,
						int *rows, int shape,
						char **error);
    SPATIALITE_DECLARE sple_dbf_field_ptr sple_alloc_dbf_field (char *name,
								unsigned char
								type,
								int offset,
								unsigned char
								length,
								unsigned char
								decimals);
    SPATIALITE_DECLARE void sple_free_dbf_field (sple_dbf_field_ptr p);
    SPATIALITE_DECLARE sple_dbf_list_ptr sple_alloc_dbf_list (char *primarykey,
							      char *geocolumn);
    SPATIALITE_DECLARE void sple_free_dbf_list (sple_dbf_list_ptr p);
    SPATIALITE_DECLARE void sple_add_dbf_field (sple_dbf_list_ptr list,
						char *name, unsigned char type,
						int offset,
						unsigned char length,
						unsigned char decimals);
    SPATIALITE_DECLARE int sple_compute_export_dbf (sple_dbf_export_ptr list,
						    int elements);
    SPATIALITE_DECLARE void sple_free_export_dbf (sple_dbf_export_ptr list,
						  int elements);
    SPATIALITE_DECLARE int sple_get_srid (unsigned char *p_blob);
    SPATIALITE_DECLARE void sple_set_srid (unsigned char *p_blob, int srid);

#ifdef __cplusplus
}
#endif

#endif				/* _SPATIALITE_H */
