/* 
/ virtualshape.h -- VirtualShape - virtual Table extension for Sqlite [shapefile]
/  
/ version 1.0, 2008 March 3
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2007  Alessandro Furieri
/
/ This library is free software; you can redistribute it and/or
/ modify it under the terms of the GNU Lesser General Public
/ License as published by the Free Software Foundation; either
/ version 2.1 of the License, or (at your option) any later version.
/
/ This library is distributed in the hope that it will be useful,
/ but WITHOUT ANY WARRANTY; without even the implied warranty of
/ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/ Lesser General Public License for more details.
/
/ You should have received a copy of the GNU Lesser General Public
/ License along with this library; if not, write to the Free Software
/ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA/
*/

#ifdef WIN32_DLL
#define VIRTUALSHAPE_DECLARE __declspec(dllexport)
#else
#define VIRTUALSHAPE_DECLARE extern
#endif

#ifndef _VIRTUALSHAPE_H
#define _VIRTUALSHAPE_H

#ifdef __cplusplus
extern "C"
{
#endif

/*
/ constants that defines byte storage order 
*/
#define VSHP_BIG_ENDIAN		0x00
#define VSHP_LITTLE_ENDIAN	0x01

/*
/ constants that defines multitype values
*/
#define VSHP_NULL_VALUE		0
#define VSHP_TEXT_VALUE		1
#define VSHP_INT_VALUE		2
#define VSHP_DOUBLE_VALUE	3
#define VSHP_BLOB_VALUE		4

/*
/ constants that defines GEOMETRY CLASSes
*/
#define VSHP_UNKNOWN			0
#define VSHP_POINT				1
#define VSHP_LINESTRING			2
#define VSHP_POLYGON			3
#define VSHP_MULTIPOINT			4
#define VSHP_MULTILINESTRING	5
#define VSHP_MULTIPOLYGON		6
#define VSHP_GEOMETRYCOLLECTION	7

/*
/ constants that defines special markers used for encoding of internal BLOB geometries 
*/
#define VSHP_MARK_START		0x00
#define VSHP_MARK_END		0xFE
#define VSHP_MARK_MBR		0x7C
#define VSHP_MARK_ENTITY	0x69

    typedef struct vshp_value_struct
    {
	/* a DBF field multitype value */
	short type;		/* the type */
	char *txt_value;	/* the text value */
	int int_value;		/* the integer value */
	double dbl_value;	/* the double value */
	void *blob_value;	/* the blob value */
	int blob_size;		/* the blob size */
    } vshp_value;
    typedef vshp_value *vshp_value_ptr;

    typedef struct vshp_dbf_field_struct
    {
	/* a DBF field definition - shapefile attribute */
	char *name;		/* field name */
	unsigned char type;	/* field type */
	int offset;		/* buffer offset [this field begins at *buffer+offset* and extends for *length* bytes */
	unsigned char length;	/* field total lenght [in bytes] */
	unsigned char decimals;	/* decimal positions */
	vshp_value_ptr value;	/* the current multitype value for this attribute */
	struct vshp_dbf_field_struct *next;	/* pointer to next element in linked list */
    } vshp_dbf_field;
    typedef vshp_dbf_field *vshp_dbf_field_ptr;

    typedef struct vshp_dbf_list_struct
    {
	/* a linked list to containt the DBF fields definitions - shapefile attributes */
	char *primary_key_name;	/* name of column used as primary key in Sqlite virtual table */
	vshp_value_ptr pk_value;	/* value for current primary key */
	char *geometry_name;	/* name of column used as geometry in Sqlite virtual table */
	vshp_value_ptr geometry;	/* value for current geometry */
	vshp_dbf_field_ptr first;	/* pointer to first element in linked list */
	vshp_dbf_field_ptr last;	/* pointer to last element in linker list */
	int seed;		/* a counter for anti-aliased column names */
    } vshp_dbf_list;
    typedef vshp_dbf_list *vshp_dbf_list_ptr;

    typedef struct vshp_vtab_struct
    {
	/* extends the sqlite3_vtab struct */
	const sqlite3_module *pModule;	/* ptr to sqlite module: USED INTERNALLY BY SQLITE */
	int nRef;		/* # references: USED INTERNALLY BY SQLITE */
	char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
	sqlite3 *db;		/* the sqlite db holding the virtual table */
	char *path;		/* the shapefile abstract path [no suffixes] */
	FILE *fl_shx;		/* the SHX file handle */
	FILE *fl_shp;		/* the SHP file handle */
	FILE *fl_dbf;		/* the DBF file handle */
	long shape;		/* the SHAPE code for the whole shapefile */
	vshp_dbf_list_ptr dbf;	/* the DBF attributes list */
	unsigned char *buf_dbf;	/* the DBF I/O buffer */
	long dbf_hdsz;		/* the DBF header length */
	int dbf_reclen;		/* the DBF record length */
	unsigned char *buf_shp;	/* the SHP I/O buffer */
	long shp_bfsz;		/* the SHP buffer current size */
    } vshp_vtab;
    typedef vshp_vtab *vshp_vtab_ptr;

    typedef struct vshp_cursor_struct
    {
	/* extends the sqlite3_vtab_cursor struct */
	vshp_vtab_ptr pVtab;	/* Virtual table of this cursor */
	long current_row;	/* the current row ID */
	int eof;		/* the EOF marker */
    } vshp_cursor;
    typedef vshp_cursor *vshp_cursor_ptr;

    typedef struct vshp_point_struct
    {
	/* an OpenGis POINT */
	double x;		/* X,Y coordinates */
	double y;
    } vshp_point;
    typedef vshp_point *vshp_point_ptr;

    typedef struct vshp_linestring_struct
    {
	/* an OpenGis LINESTRING */
	int points;		/* number of vertices */
	vshp_point_ptr geometry;	/* pointer referencing an array of POINTs */
	double minx;		/* MBR - minimun x */
	double miny;		/* MBR - minimun y */
	double maxx;		/* MBR - maximun x */
	double maxy;		/* MBR - maximun y */
    } vshp_linestring;
    typedef vshp_linestring *vshp_linestring_ptr;

    typedef struct vshp_polygon_struct
    {
	/* an OpenGis POLYGON */
	vshp_linestring_ptr external;	/* pointer referenging the polygon's EXTERNAL border */
	int borders;		/* number of interneal borders */
	vshp_linestring_ptr *internals;	/* pointer referencing an array of polygon's INTERNAL borders [may be null ... if no internal border exists] */
	double minx;		/* MBR - minimun x */
	double miny;		/* MBR - minimun y */
	double maxx;		/* MBR - maximun x */
	double maxy;		/* MBR - maximun y */
    } vshp_polygon;
    typedef vshp_polygon *vshp_polygon_ptr;

    typedef struct vshp_generic_geometry_struct
    {
	/* a generic container for OpenGis GEOMETRIES */
	short type;		/* geometry type [may be vshp_POINT,vshp_LINESTRING or vshp_POLYGON] */
	void *geometry;
	/* should be safely casted to *vshp_point_ptr if *type* = vshp_POINT
	   / to *vshp_linestring_ptr* if *type* = vshp_LINESTRING 
	   / or to *vshp_polygon_ptr* if *type* = vshp_POLYGON
	 */
    } vshp_generic_geometry;
    typedef vshp_generic_geometry *vshp_generic_geometry_ptr;

    typedef struct vshp_geometry_struct
    {
	/* an OpenGis GEOMETRY COLLECTION */
	int little_endian;	/* TRUE if buffer uses little-endian encoding */
	const unsigned char *blob;	/* pointer referencing the WKB encoded buffer */
	int size;		/* buffer size */
	int entities;		/* number of entities */
	vshp_generic_geometry_ptr *geometries;	/* pointer referenging an array of generic geometries */
	double minx;		/* MBR - minimun x */
	double miny;		/* MBR - minimun y */
	double maxx;		/* MBR - maximun x */
	double maxy;		/* MBR - maximun y */
    } vshp_geometry;
    typedef vshp_geometry *vshp_geometry_ptr;

#ifdef __cplusplus
}
#endif

#endif				/* _VIRTUALSHAPE_H */
